%%
% LPV Example
%
% M-file to analyze an uncetrain linear parameter varying system using IQCs 
% and dissipation inequalities. The analysis is performed for various bounds
% on the parameter variation rate and compared to the wcgain command
% The LPV system is taken from "Time-Delay System", Weehon Tan, Berkeley, 
% Automatica vol33, 1997, pp411-419
% In order to run the example Yalmip and SDPT3 are required


%% 
% Setting up the plant model
clear all
yvec = (2:1:7); %scheduling parameter vector
K = sqrt(4.8*yvec-8.6); %parameter varying gain of the 1st order LPV system
T = sqrt(133.6-16.8*yvec); %time constant

% Adding a basic gain scheduled PI controller
zeta = .7; %CL damping requirement at each grid point
omega = .25; %CL frequency requirement at each grid point

Ki = omega.^2.*T./K; %Integrator gain
Kp = (2*zeta*omega.*T-1)./K; %proportinal gain

%build LPV plant, controller and closed loop
for ii = 1:length(yvec)
    P(:,:,ii)= ss(-1/T(ii),1/(T(ii)),K(ii),0);
    
    C(:,:,ii) = ss(0,Ki(ii),1,Kp(ii));
    
    clp(:,:,ii) = feedback(P(:,:,ii)*C(:,:,ii),1);
    
end

%time delay at the plant intput
Td = .5;
tdelay = tf([Td^2/12,-Td/2,1],[Td^2/12,Td/2,1]);


%% quadratic P, rate bounds [-2,2] increase sector [0,beta]
clear giqclmi

RateBounds = [-2,2];
% basis function for the parameter dependent lyapunov function
Fbasis = ones(3,1,6); %First term is constant
Fbasis(2,1,:) = reshape(yvec',1,1,6); %second term linear in rho
Fbasis(3,1,:) = reshape(yvec'.^2,1,1,6); %third term quadratic in rho
%gradient of the basis function with respect to the parameters
Fgrad = zeros(2,1,6);
Fgrad(2,1,:) = ones(1,1,6);
Fgrad(3,1,:) = reshape(2*yvec',1,1,6);

% sector bound [alpha, beta]
alpha = 0;
betavec = [0.1,0.2,0.3,0.4,0.5,0.6];

for ii = 1:length(betavec)
    beta = betavec(ii);
    
    %build plant interconnection
    plant = P;
    controller = C;
    systemnames = 'plant controller tdelay';
    inputvar = '[ v{1}; d{1} ]';
    outputvar = '[ controller; -plant(1) ]';
    input_to_plant = '[tdelay+d(1)]';
    input_to_tdelay = '[v + controller]';
    input_to_controller = '[-plant(1) ]';
    sysoutname = 'G';
    cleanupsysic = 'yes';
    sysic    
    
    %sector IQC
    M1 = [-2*alpha*beta,alpha+beta;alpha+beta,-2];
    s=tf('s');
    psi1 =blkdiag(1/(s+1),1);

    %Popov IQC
    PI2 = -[0, (s+1e-4)/(s+1); (s-1e-4)/(s-1), 0];
    PI2 = PI2 + diag(0.01*[beta^2,-1]);
    [psi2, M2] = jspec(PI2);
   
    %Zames Falb IQC
    H = tf(1,[1,.1]);
    PI3 = minreal(ss([0, 1+H; 1+H', -2-(H+H')]),[],0);
    PI3 = PI3 + diag(0.01*[beta^2,-1]);
    [psi3,M3] = jspec(PI3);
    psi3 = psi3*psi1;
    
    %analysis for sector bound
    IQClist = {{psi1,M1}};
    [giqclmi(1,ii),info] = simplesectboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds);
    
   
    %analysis for static sector bound
    IQClist = {{psi1,M1},{psi2,M2}};
    [giqclmi(2,ii),info] = simplesectboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds);
    
    %analysis for monotonic, static sector bound
    IQClist = {{psi1,M1},{psi2,M2},{psi3,M3}};
    [giqclmi(3,ii),info] = simplesectboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds);
       
end


%plot results
figure(2)
hold on
plot(betavec,giqclmi(1,:),'+--','color',mycolor('maroon'),'MarkerSize',12)
plot(betavec,giqclmi(2,:),'x-','color',mycolor('coolblue'),'MarkerSize',12)
plot(betavec,giqclmi(3,:),'o:','color',mycolor('darkgray'),'MarkerSize',8)
xlim([0 0.8])
xlabel('sector bound \beta [-]')
ylabel('worst case gain \gamma [-]')
legend('sector bounded','static sector bounded','monotonic, static sector bounded');
if exist('garyfyFigure','file')
    garyfyFigure;
end
box on
legend('Location','NorthWest')


